/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

//this ile requires "wstp.h" for compilation

#include "autofitmathematica.h"

AutoFitMathematica::AutoFitMathematica(DataStorage *dst, SettingsStorage *sst, short dataThinningFactor){

    env = WSInitialize(0);

    QString *WSopenCommand = new QString("-linkname \"");
    WSopenCommand->append(*(sst->prgset->kernelPath));
    WSopenCommand->append(" -wstp -noprompt\" -linklaunch");
    link = WSOpenString(env, WSopenCommand->toUtf8().data(), &err);
    delete WSopenCommand;
    WSActivate(link);

    QFile *file = new QFile(*(dst->applicationDirPath) + "/data/fitFunct.txt");
    if (file->open(QIODevice::ReadOnly | QIODevice::Text)){
        QTextStream *in = new QTextStream(file);
        while(!(in->atEnd())){
            WSPutFunction(link, "EnterTextPacket", 1);
            WSPutString(link, in->readLine().toUtf8().data());
            WSEndPacket(link);
            WSFlush(link);
        }
        delete in;
    }
    delete file;

    WSPutFunction(link, "EnterTextPacket", 1);
    WSPutString(link, QString("lambda=" + QString::number(dst->prgpar->currentWavelength->value,'f',9) + ";").toUtf8().data());
    WSEndPacket(link);
    WSFlush(link);


    if (dst->prgpar->useQ){
        WSPutFunction(link, "EnterTextPacket", 1);
        WSPutString(link, "IincC[b_,s_]:=IcomC[s] Recoil[s] Qabs[s] Q[b,s]");
        WSEndPacket(link);
        WSFlush(link);

    }

    if (dst->prgpar->useA){
        WSPutFunction(link, "EnterTextPacket", 1);
        WSPutString(link, QString("d=" + QString::number(dst->prgpar->par_d,'f',5) + ";").toUtf8().data());
        WSEndPacket(link);
        WSFlush(link);
        WSPutFunction(link, "EnterTextPacket", 1);
        WSPutString(link, QString("mue=" + QString::number(dst->pars->getParameterValue(Enumerations::mue_ab, Enumerations::val),'f',5) + ";").toUtf8().data());
        WSEndPacket(link);
        WSFlush(link);
    }
    if (dst->prgpar->useCorrAutoColl){
        WSPutFunction(link, "EnterTextPacket", 1);
        WSPutString(link, QString("R=" + QString::number(dst->prgpar->par_r,'f',5) + ";").toUtf8().data());
        WSEndPacket(link);
        WSFlush(link);
        WSPutFunction(link, "EnterTextPacket", 1);
        WSPutString(link, QString("delta=" + QString::number(dst->prgpar->par_delta,'f',5) + ";").toUtf8().data());
        WSEndPacket(link);
        WSFlush(link);
        WSPutFunction(link, "EnterTextPacket", 1);
        WSPutString(link, QString("l=" + QString::number(dst->prgpar->par_l,'f',5) + ";").toUtf8().data());
        WSEndPacket(link);
        WSFlush(link);
    }

    if (dst->prgpar->useA && dst->prgpar->useGradient && dst->prgpar->useCorrAutoColl){
        WSPutFunction(link, "EnterTextPacket", 1);
        WSPutString(link, "Ifit[b_,k_,cn_,mu_,Nm_,a3min_,da3_,sig3_,u3_,eta_,nu_,lm_,lcc_,sig1_,q_,cN_,cO_,dan_,gp_,s_]:=k*P[s]*(Icoh[cn,mu,Nm,a3min,da3,sig3,u3,eta,nu,lm,lcc,sig1,q,cN,cO,dan,s]+IincC[b,s])*A[ArcSin[(s*lambda/2)]*(2/Degree),d,mue]*T[gp,s]/F[lambda,R,delta,l,s]");
        WSEndPacket(link);
        WSFlush(link);
    }
    else
        if (dst->prgpar->useA && dst->prgpar->useGradient){
            WSPutFunction(link, "EnterTextPacket", 1);
            WSPutString(link, "Ifit[b_,k_,cn_,mu_,Nm_,a3min_,da3_,sig3_,u3_,eta_,nu_,lm_,lcc_,sig1_,q_,cN_,cO_,dan_,gp_,s_]:=k*P[s]*(Icoh[cn,mu,Nm,a3min,da3,sig3,u3,eta,nu,lm,lcc,sig1,q,cN,cO,dan,s]+IincC[b,s])*A[ArcSin[(s*lambda/2)]*(2/Degree),d,mue]*T[gp,s]");
            WSEndPacket(link);
            WSFlush(link);
        }
        else
            if (dst->prgpar->useA && dst->prgpar->useCorrAutoColl){
                WSPutFunction(link, "EnterTextPacket", 1);
                WSPutString(link, "Ifit[b_,k_,cn_,mu_,Nm_,a3min_,da3_,sig3_,u3_,eta_,nu_,lm_,lcc_,sig1_,q_,cN_,cO_,dan_,gp_,s_]:=k*P[s]*(Icoh[cn,mu,Nm,a3min,da3,sig3,u3,eta,nu,lm,lcc,sig1,q,cN,cO,dan,s]+IincC[b,s])*A[ArcSin[(s*lambda/2)]*(2/Degree),d,mue]/F[lambda,R,delta,l,s]");
                WSEndPacket(link);
                WSFlush(link);
            }
            else
                if (dst->prgpar->useGradient && dst->prgpar->useCorrAutoColl){
                    WSPutFunction(link, "EnterTextPacket", 1);
                    WSPutString(link, "Ifit[b_,k_,cn_,mu_,Nm_,a3min_,da3_,sig3_,u3_,eta_,nu_,lm_,lcc_,sig1_,q_,cN_,cO_,dan_,gp_,s_]:=k*P[s]*(Icoh[cn,mu,Nm,a3min,da3,sig3,u3,eta,nu,lm,lcc,sig1,q,cN,cO,dan,s]+IincC[b,s])*T[gp,s]/F[lambda,R,delta,l,s]");
                    WSEndPacket(link);
                    WSFlush(link);
                }
                else
                    if (dst->prgpar->useA){
                        WSPutFunction(link, "EnterTextPacket", 1);
                        WSPutString(link, "Ifit[b_,k_,cn_,mu_,Nm_,a3min_,da3_,sig3_,u3_,eta_,nu_,lm_,lcc_,sig1_,q_,cN_,cO_,dan_,gp_,s_]:=k*P[s]*(Icoh[cn,mu,Nm,a3min,da3,sig3,u3,eta,nu,lm,lcc,sig1,q,cN,cO,dan,s]+IincC[b,s])*A[ArcSin[(s*lambda/2)]*(2/Degree),d,mue]");
                        WSEndPacket(link);
                        WSFlush(link);
                    }
                    else
                        if (dst->prgpar->useGradient){
                            WSPutFunction(link, "EnterTextPacket", 1);
                            WSPutString(link, "Ifit[b_,k_,cn_,mu_,Nm_,a3min_,da3_,sig3_,u3_,eta_,nu_,lm_,lcc_,sig1_,q_,cN_,cO_,dan_,gp_,s_]:=k*P[s]*(Icoh[cn,mu,Nm,a3min,da3,sig3,u3,eta,nu,lm,lcc,sig1,q,cN,cO,dan,s]+IincC[b,s])*T[gp,s]");
                            WSEndPacket(link);
                            WSFlush(link);
                        }
                        else
                            if (dst->prgpar->useCorrAutoColl){
                                WSPutFunction(link, "EnterTextPacket", 1);
                                WSPutString(link, "Ifit[b_,k_,cn_,mu_,Nm_,a3min_,da3_,sig3_,u3_,eta_,nu_,lm_,lcc_,sig1_,q_,cN_,cO_,dan_,gp_,s_]:=k*P[s]*(Icoh[cn,mu,Nm,a3min,da3,sig3,u3,eta,nu,lm,lcc,sig1,q,cN,cO,dan,s]+IincC[b,s])/F[lambda,R,delta,l,s];");
                                WSEndPacket(link);
                                WSFlush(link);
                            }

    WSPutFunction(link, "EnterTextPacket", 1);
    WSPutString(link, "logProbecs={};");
    WSEndPacket(link);
    WSFlush(link);

    for (int i = 0; i < dst->data_for_ChooseTemplate_AutoFit->size(); i += (1 * dataThinningFactor)){
        WSPutFunction(link, "EnterTextPacket", 1);
        WSPutString(link, QString("logProbecs=Insert[logProbecs,{" + QString::number(dst->data_for_ChooseTemplate_AutoFit->at(i).x(),'f',12) + "," + QString::number(dst->data_for_ChooseTemplate_AutoFit->at(i).y(),'f',12) + "},-1];").toUtf8().data());
        WSEndPacket(link);
        WSFlush(link);
    }

    if ( ( (dst->data_for_ChooseTemplate_AutoFit->size() - 1) % dataThinningFactor ) != 0){ //to ensure last data point is used
        WSPutFunction(link, "EnterTextPacket", 1);
        WSPutString(link, QString("logProbecs=Insert[logProbecs,{" + QString::number(dst->data_for_ChooseTemplate_AutoFit->at(dst->data_for_ChooseTemplate_AutoFit->size() - 1).x(),'f',12) + "," + QString::number(dst->data_for_ChooseTemplate_AutoFit->at(dst->data_for_ChooseTemplate_AutoFit->size() - 1).y(),'f',12) + "},-1];").toUtf8().data());
        WSEndPacket(link);
        WSFlush(link);
    }
}


AutoFitMathematica::~AutoFitMathematica(){

}

void AutoFitMathematica::autoFit(ParameterSetAutoFit *pars, AutoFitSequence *afseq, int step, QStringList *parsNames, double b, double cN, double cO){
    for (int i = ( (step > 0) ? (step-1) : 0); i < ( (step > 0) ? (step) : (afseq->numberOfSteps) ); ++i)

        if (afseq->autoFitSteps->at(i).useStep || step > 0){

            QString *fitCommand = new QString("autofit = NonlinearModelFit[logProbecs,{Log[10,Ifit[");
            fitCommand->append(QString::number(b,'f',9) + ",");


            //no loop for this part as order of commands is crucial and pars order differs in Enumerations

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::kAF].autoFit)
                fitCommand->append("k,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::kAF),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::cnoAF].autoFit)
                fitCommand->append("cn,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::cnoAF),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::muAF].autoFit)
                fitCommand->append("mu,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::muAF),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::muAF].autoFit && afseq->autoFitSteps->at(i).autoFitElements[Enumerations::betaAF].autoFit)
                fitCommand->append("mu/beta,");
            else
                if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::muAF].autoFit)
                    fitCommand->append("mu/"+ QString::number(pars->getParameterValue(Enumerations::betaAF),'f',9) + ",");
                else
                    if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::betaAF].autoFit)
                        fitCommand->append(QString::number(pars->getParameterValue(Enumerations::muAF),'f',9) + "/beta,");
                    else
                        fitCommand->append(QString::number((pars->getParameterValue(Enumerations::muAF))/(pars->getParameterValue(Enumerations::betaAF)),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::a3AF].autoFit && afseq->autoFitSteps->at(i).autoFitElements[Enumerations::da3AF].autoFit)
                fitCommand->append("a3-da3,");
            else
                if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::a3AF].autoFit)
                    fitCommand->append("a3-"+ QString::number(pars->getParameterValue(Enumerations::da3AF),'f',9) + ",");
                else
                    if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::da3AF].autoFit)
                        fitCommand->append(QString::number(pars->getParameterValue(Enumerations::a3AF),'f',9) + "-da3,");
                    else
                        fitCommand->append(QString::number((pars->getParameterValue(Enumerations::a3AF))-(pars->getParameterValue(Enumerations::da3AF)),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::da3AF].autoFit)
                fitCommand->append("da3,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::da3AF),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::sig3AF].autoFit)
                fitCommand->append("sig3,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::sig3AF),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::u3AF].autoFit)
                fitCommand->append("u3,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::u3AF),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::etaAF].autoFit)
                fitCommand->append("eta,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::etaAF),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::nuAF].autoFit)
                fitCommand->append("nu,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::nuAF),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::nuAF].autoFit && afseq->autoFitSteps->at(i).autoFitElements[Enumerations::alphaAF].autoFit)
                fitCommand->append("nu/alpha,");
            else
                if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::nuAF].autoFit)
                    fitCommand->append("nu/"+ QString::number(pars->getParameterValue(Enumerations::alphaAF),'f',9) + ",");
                else
                    if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::alphaAF].autoFit)
                        fitCommand->append(QString::number(pars->getParameterValue(Enumerations::nuAF),'f',9) + "/alpha,");
                    else
                        fitCommand->append(QString::number((pars->getParameterValue(Enumerations::nuAF))/(pars->getParameterValue(Enumerations::alphaAF)),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::lccAF].autoFit)
                fitCommand->append("lcc,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::lccAF),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::sig1AF].autoFit)
                fitCommand->append("sig1,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::sig1AF),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::qAF].autoFit)
                fitCommand->append("q,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::qAF),'f',9) + ",");

            fitCommand->append(QString::number(cN,'f',9) + ",");
            fitCommand->append(QString::number(cO,'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::danAF].autoFit)
                fitCommand->append("dan,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::danAF),'f',9) + ",");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::gAF].autoFit)
                fitCommand->append("gp,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::gAF),'f',9) + ",");

            fitCommand->append("s]+");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::const1AF].autoFit)
                fitCommand->append("const1]+");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::const1AF),'f',9) + "]+");

            if (afseq->autoFitSteps->at(i).autoFitElements[Enumerations::const2AF].autoFit)
                fitCommand->append("const2,");
            else
                fitCommand->append(QString::number(pars->getParameterValue(Enumerations::const2AF),'f',9) + ",");


            fitCommand->append("{");

            for (int j = 0; j < Enumerations::countOf_parNameAutoFit; ++j)
                if (afseq->autoFitSteps->at(i).autoFitElements[j].autoFit){
                    if (afseq->autoFitSteps->at(i).autoFitElements[j].useMin && afseq->autoFitSteps->at(i).autoFitElements[j].useMax)
                        fitCommand->append(QString::number(afseq->autoFitSteps->at(i).autoFitElements[j].min,'f',9) + "<=" + parsNames->at(j)+ "<=" + QString::number(afseq->autoFitSteps->at(i).autoFitElements[j].max,'f',9) + ",");
                    else
                        if (afseq->autoFitSteps->at(i).autoFitElements[j].useMin)
                            fitCommand->append(QString::number(afseq->autoFitSteps->at(i).autoFitElements[j].min,'f',9) + "<=" + parsNames->at(j) + ",");
                        else
                            if (afseq->autoFitSteps->at(i).autoFitElements[j].useMax)
                                fitCommand->append(parsNames->at(j) + "<=" + QString::number(afseq->autoFitSteps->at(i).autoFitElements[j].max,'f',9) + ",");
                }


            if (afseq->autoFitSteps->at(i).additionalConditions->size() > 0)
                for (int j = 0; j < afseq->autoFitSteps->at(i).additionalConditions->size(); ++j)
                    fitCommand->append(afseq->autoFitSteps->at(i).additionalConditions->at(j) + ",");

            fitCommand->chop(1); //remove last comma


            fitCommand->append("}},{");

            for (int j = 0; j < Enumerations::countOf_parNameAutoFit; ++j)
                if (afseq->autoFitSteps->at(i).autoFitElements[j].autoFit){
                    if ( (pars->getParameterValue(Enumerations::parNameAutoFit(j)) >= afseq->autoFitSteps->at(i).autoFitElements[j].min) && (pars->getParameterValue(Enumerations::parNameAutoFit(j)) <= afseq->autoFitSteps->at(i).autoFitElements[j].max) )
                        fitCommand->append("{" + parsNames->at(j) + ", "+ QString::number(pars->getParameterValue(Enumerations::parNameAutoFit(j)),'f',9) + "},");
                    else
                        if (pars->getParameterValue(Enumerations::parNameAutoFit(j)) < afseq->autoFitSteps->at(i).autoFitElements[j].min)
                            fitCommand->append("{" + parsNames->at(j) + ", "+ QString::number(afseq->autoFitSteps->at(i).autoFitElements[j].min,'f',9) + "},");
                        else if (pars->getParameterValue(Enumerations::parNameAutoFit(j)) > afseq->autoFitSteps->at(i).autoFitElements[j].max)
                            fitCommand->append("{" + parsNames->at(j) + ", "+ QString::number(afseq->autoFitSteps->at(i).autoFitElements[j].max,'f',9) + "},");
                }


            fitCommand->chop(1); //remove last comma
            fitCommand->append("},s][\"BestFitParameters\"];");

            //std::cout << fitCommand->toUtf8().data() << std::endl; //TEST

            WSPutFunction(link, "EnterTextPacket", 1);
            WSPutString(link, fitCommand->toUtf8().data());
            WSEndPacket(link);
            WSFlush(link);

            delete fitCommand;


            const char *string = 0;
            QString *returned = 0;

            for (int j = 0; j < Enumerations::countOf_parNameAutoFit; ++j)
                if (afseq->autoFitSteps->at(i).autoFitElements[j].autoFit){
                    WSPutFunction(link, "EnterTextPacket", 1);
                    QString *textPacket = new QString (parsNames->at(j) + "/.autofit");
                    WSPutString(link, textPacket->toUtf8().data());
                    delete textPacket;
                    WSEndPacket(link);
                    WSFlush(link);
                    string = 0;
                    while( (pkt = WSNextPacket(link) ) && pkt != RETURNTEXTPKT)
                        WSNewPacket(link);
                    WSGetString(link, &string);
                    returned = new QString(QString::fromUtf8(string));
                    WSReleaseString(link, string);
                    pars->setParameterValue(Enumerations::parNameAutoFit(j), returned->toDouble());
                    delete returned;
                }

        }

    //close the MathLink connection when done
    WSPutFunction(link, "EnterTextPacket", 1);
    WSPutString(link,"Exit");
    WSEndPacket(link);
    WSFlush(link);
    WSClose(link);
    WSDeinitialize(env);

}

