/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef DATACALCULATIONS_H
#define DATACALCULATIONS_H

#include <cmath>

#include <QPointF>
#include <QVector>

#include "calculations.h"
#include "enumerations.h"
#include "parameterset.h"

#include <QObject>

class DataCalculations : public QObject{

    Q_OBJECT

public:
    explicit DataCalculations(QString *dataPath = 0, QWidget *parentWidget = 0, QObject *parent = 0);
    ~DataCalculations();

    void dataConversion(QVector<QPointF> *data, Enumerations::XFormat XFormat, Enumerations::YFormat YFormat, double wavelength); //converts measurement-data into the format "intensity (linear) versus 2theta [degrees]"
    void calcDataCurve(QVector<QPointF> *data_cor, QVector<QPointF> *data, bool corrPosition, Enumerations::corrPositionMethod corrPositionMethod, bool corrIntensity, Enumerations::corrIntensityMethod corrIntensityMethod, ParameterSet *pars, double par_r, double par_d, double wavelength, QVector<QPointF> **data_for_ChooseTemplate_AutoFit, double plotMinX, double plotMaxX);
    void calcFitCurve(QVector<QPointF> *target, ParameterSet *pars, double min, double max, int thinningFactor, double wavelength, bool useA, double par_d, bool useGradient, bool useCorrAutoColl, double par_r, double par_delta, double par_l, bool useQ, double par_b);
    void calcDeviationData(QVector<QPointF> *target, QVector<QPointF> *data, ParameterSet *pars, double wavelength, bool useA, double par_d, bool useGradient, bool useCorrAutoColl, double par_r, double par_delta, double par_l, bool useQ, double par_b);

private:
    QWidget *parent;
    Calculations *calc;

};

#endif // DATACALCULATIONS_H
