/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "datastorage.h"

DataStorage::DataStorage(QString *applicationDirPath, QWidget *parentWidget, QObject *parent) : QObject(parent){

    this->parent = parentWidget;


    dfa = new DataFileAccess(this->parent);
    ifa = new INIFileAccess(this->parent);

    this->applicationDirPath = applicationDirPath;


    wavelengths = new QVector<wavelength>(0);

    prgpar = new ProgramParameterSet(this->parent);

    parsPath = 0;
    pars = new ParameterSet(this->parent);
    parsAutoFit = 0; //to be set before showing AutoFitSettingsWindow
    plotPars_MainWindow = pars;
    plotPars_Deviation = pars;
    parsMeaning = new QVector<parsMean>(0);
    parsType = new QVector<QString>(0);
    parsType->append("val");
    parsType->append("min");
    parsType->append("max");
    parsType->append("dec");
    parsType->append("inc");
    parsNamesAutoFit = new QStringList();

    dataPath = 0;
    //initialisation of QVectors used for storage of data/fit points
    data = new QVector<QPointF>(0);
    data_cor = new QVector<QPointF>(0);
    data_for_ChooseTemplate_AutoFit = new QVector<QPointF>(0);
    fit = new QVector<QPointF>(0);

    deviationData = new QVector<QPointF>(0);

    for (int i = 0; i < number_of_templates; ++i)
        fitTemplate[i] =  new QVector<QPointF>(0);
    chosenTemplate = 0; //0 means no template chosen

}

DataStorage::~DataStorage(){

    //clear template data
    for (int i = 0; i < number_of_templates; ++i){
        fitTemplate[i]->clear();
        delete fitTemplate[i];
    }

    //clear and delete QVectors used for storage of data/fit/deviation points
    deviationData->clear();
    delete deviationData;
    fit->clear();
    delete fit;
    data_for_ChooseTemplate_AutoFit->clear();
    delete data_for_ChooseTemplate_AutoFit;
    data_cor->clear();
    delete data_cor;
    data->clear();
    delete data;
    delete dataPath;

    parsNamesAutoFit->clear();
    delete parsNamesAutoFit;
    parsType->clear();
    delete parsType;
    parsMeaning->clear();
    delete parsMeaning;
    delete parsAutoFit;
    delete pars;
    delete parsPath;

    delete prgpar;

    wavelengths->clear();
    delete wavelengths;

    delete ifa;
    delete dfa;

    delete applicationDirPath;

}

bool DataStorage::readParsMeaning(){
    return dfa->readParsMeaning(applicationDirPath, parsMeaning);
}

bool DataStorage::readParsNamesAutoFit(){
    return dfa->readParsNamesAutoFit(applicationDirPath, parsNamesAutoFit);
}

bool DataStorage::readParsDefaultValues(ParameterSet *target){
    return ifa->readParsDefaultValues(target, applicationDirPath, parsMeaning, parsType);
}

Enumerations::errorType DataStorage::loadDataFile(QString *defaultDir){
    return (dfa->loadXYFile(data, &dataPath, new QString(tr("Messwerte laden")), new QString(tr("Messwerte")), defaultDir));
}

bool DataStorage::readDataFile(){
    return (dfa->readXYFile(data, dataPath));
}

Enumerations::errorType DataStorage::saveXYFile(Enumerations::dataToWrite dataToWrite, QString *defaultDir, QWidget *parent){
    switch (dataToWrite){
        case Enumerations::DataCurve :
            return dfa->saveXYFile(data_cor, new QString(tr("aktuelle Messkurve als X-Y-Datei speichern")), defaultDir, parent);
            break;
        case Enumerations::FitCurve :
            return dfa->saveXYFile(fit, new QString(tr("aktuelle Anpassungskurve als X-Y-Datei speichern")), defaultDir, parent);
            break;
        case Enumerations::DeviationCurve :
            return dfa->saveXYFile(deviationData, new QString(tr("aktuelle Fehlerkurve als X-Y-Datei speichern")), defaultDir, parent);
            break;
        default :
            return Enumerations::OtherError;
            break;
    }
    return Enumerations::OtherError;
}

bool DataStorage::saveImageFile(QPixmap *imageDataToWrite, QString *dialogCaption, QString *defaultDir, QWidget *parent){
    return dfa->saveImageFile(imageDataToWrite, dialogCaption, defaultDir, parent);
}

int DataStorage::readWavelengths(bool skipUserFile){
    return dfa->readWavelengths(applicationDirPath, wavelengths, skipUserFile);
}

void DataStorage::writeWavelengths(int selectedWavelength){
    dfa->writeWavelengths(applicationDirPath, selectedWavelength, wavelengths);
}
