/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef DATASTORAGE_H
#define DATASTORAGE_H

#include <QPointF>
#include <QString>
#include <QStringList>
#include <QVector>

#include "datafileaccess.h"
#include "enumerations.h"
#include "inifileaccess.h"
#include "parameterset.h"
#include "parametersetautofit.h"
#include "programparameterset.h"
#include "structs.h"

#include <QObject>

class DataStorage : public QObject{

    Q_OBJECT

public:
    //all only internal stored values are initialised with default values in constructor
    explicit DataStorage(QString *applicationDirPath = 0, QWidget *parentWidget = 0, QObject *parent = 0);
    ~DataStorage();


    //most variables are public for easier access

    QString *applicationDirPath; //path to CarbX.exe

    QVector<wavelength> *wavelengths; //selectable wavelengths

    ProgramParameterSet *prgpar;

    QString             *parsPath; //path to loaded parameter file, 0 if no file loaded
    ParameterSet        *pars;
    ParameterSetAutoFit *parsAutoFit;
    ParameterSet        *plotPars_MainWindow;
    ParameterSet        *plotPars_Deviation;
    QVector<parsMean>   *parsMeaning;
    QVector<QString>    *parsType;
    QStringList         *parsNamesAutoFit; //parameter names for Mathematica in order of Enumerations::parNameAutoFit

    QString          *dataPath;                               //path to loaded data file, 0 if no file loaded
    QVector<QPointF> *data;                                   //original data as read from file
    QVector<QPointF> *data_cor;                               //corrected data for plotting
    QVector<QPointF> *data_for_ChooseTemplate_AutoFit;        //copy of corrected data for chooseTemplate-Dialog and autoFit
    QVector<QPointF> *fit;                                    //calculated fit

    QVector<QPointF> *deviationData; //calculated deviation

    static const int number_of_templates = 8; //can be adapted if GUI is adapted accordingly
    QVector<QPointF> *fitTemplate[number_of_templates];
    short chosenTemplate;


    //public methods

    bool readParsMeaning();
    bool readParsDefaultValues(ParameterSet *target);
    bool readParsNamesAutoFit();

    Enumerations::errorType loadDataFile(QString *defaultDir); //loads data file from hard disk
    bool readDataFile(); //reads data file from disk, uses dataPath as filePath
    Enumerations::errorType saveXYFile(Enumerations::dataToWrite dataToWrite, QString *defaultDir, QWidget *parent); //writes XY-File to hard disk

    bool saveImageFile(QPixmap *imageDataToWrite, QString *dialogCaption, QString *defaultDir, QWidget *parent);  //save QPixmap to image file

    int readWavelengths(bool skipUserFile);
    void writeWavelengths(int selectedWavelength);


private:
    QWidget *parent;
    DataFileAccess *dfa;
    INIFileAccess  *ifa;

};

#endif // DATASTORAGE_H
