/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef DEVIATION_H
#define DEVIATION_H

#include <QBrush>
#include <QCloseEvent>
#include <QColor>
#include <QDesktopWidget>
#include <QKeyEvent>
#include <QPen>
#include <QPixmap>
#include <QSize>
#include <QShowEvent>
#include <QString>

#include <windows.h>

#include <qwt_curve_fitter.h>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_grid.h>
#include <qwt_series_data.h>
#include <qwt_symbol.h>

#include "datacalculations.h"
#include "datastorage.h"
#include "enumerations.h"
#include "settingsstorage.h"

#include <QMainWindow>

class MainWindow;

namespace Ui {
    class Deviation;
}

class Deviation : public QMainWindow{

    Q_OBJECT

public:
    explicit Deviation(MainWindow *mw = 0, DataCalculations *datacal = 0, DataStorage *dst = 0, SettingsStorage *sst = 0, QWidget *parent = 0);
    ~Deviation();

    void setCloseAllowed();

    //set method for state of checkbox to enable/disable tracking of changes  MainWindow's plotWidget
    void setFollowChanges(bool value);

    //set widgets for plotUpdate enabled/disabled if autoFit results are displayed
    void setEnabledDisabledForDisplayOfAutoFitResults(bool displayAutoFitResults);

    //set method for state of checkbox to enable/disable display of interpolation curve
    void setInterpolation(bool value);

    void updatePlot();
    void resetPlot();

    //attaches/detaches curves in correct order and corresponding to state of checkBox_Interpolation
    //applies settings of thickness
    void updateCurves();

    void updateBorders();


private slots:
    void on_scrollBarH_valueChanged(int value);
    void on_scrollBarV_valueChanged(int value);
    void on_checkBox_Interpolation_clicked(bool checked);
    void on_checkBox_followChanges_clicked(bool checked);
    void on_button_update_clicked();
    void on_button_SavePlotImage_clicked();
    void on_button_SavePlotAsXY_clicked();
    void on_button_Close_clicked();

private:
    Ui::Deviation *ui;

    QwtPlotGrid *grid;

    QColor *colorErr;
    QBrush *brushErr;
    QPen *penErr;
    QSize *sizeErr;
    QwtSymbol *symbolErr;
    QwtPlotCurve *curveErr;

    QColor *colorInterpol;
    QPen *penInterpol;
    QwtPlotCurve *curveInterpol;

    bool canClose; //close allowed? (close only by parent)

    MainWindow       *mw;      //pointer to MainWindow  - must not be deleted
    DataCalculations *datacal; //passed from MainWindow - no need to delete
    DataStorage      *dst;     //passed from MainWindow - no need to delete
    SettingsStorage  *sst;     //passed from MainWindow - no need to delete

    void setSize();

    //Events
    void showEvent(QShowEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void closeEvent(QCloseEvent *event);

};

#endif // DEVIATION_H
