/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "dialog_choosetemplate.h"
#include "ui_dialog_choosetemplate.h"

Dialog_chooseTemplate::Dialog_chooseTemplate(DataCalculations *datacal, DataStorage *dst, SettingsStorage *sst, ParameterFileAccess *pfa, QVector<QPointF> *data, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Dialog_chooseTemplate){

    ui->setupUi(this);

    //own code

    this->datacal = datacal;
    this->dst = dst;
    this->sst = sst;
    this->pfa = pfa;
    this->data = data;

    this->setFixedSize(this->width(), this->height());
    this->setWindowFlags(Qt::MSWindowsFixedSizeDialogHint);
    this->setWindowFlags(Qt::CustomizeWindowHint);
    setFocus();

    canClose = false;

    if (sst->prgset->calcTemplates)
        ui->button_calculateTemplates->hide();

    ui->plot_data->enableAxis(QwtPlot::yLeft, false);
    ui->plot_data->enableAxis(QwtPlot::xBottom, false);
    curveData = new QwtPlotCurve("data");
    curveData->setStyle(QwtPlotCurve::Dots);
    colorData = new QColor(Qt::blue);
    brushData = new QBrush (*colorData);
    penData = new QPen(*colorData);
    sizeData = new QSize(this->sst->prgset->thicknessOfCurveDots,this->sst->prgset->thicknessOfCurveDots);
    symbolData = new QwtSymbol(QwtSymbol::Ellipse, *brushData, *penData, *sizeData);
    curveData->setSymbol(symbolData);
    curveData->setData(new QwtPointSeriesData(*data));
    curveData->attach(ui->plot_data);
    ui->plot_data->replot();

    colorTemplate = new QColor(Qt::red);
    penTemplate = new QPen(*colorTemplate);
    penTemplate->setWidth(this->sst->prgset->thicknessOfCurveLines);

    QString *warning = new QString(""); //warn user if wavelength or sample thickness from read template with current settings

    for (int i = 0; i < dst->number_of_templates; ++i){
        curveTemplate[i] = new QwtPlotCurve("template" + QString::number(i));
        curveTemplate[i]->setStyle(QwtPlotCurve::Lines);
        curveTemplate[i]->setPen(*penTemplate);
        curveTemplate[i]->setCurveFitter(new QwtSplineCurveFitter());
        ParameterSet *pars = new ParameterSet(this);
        bool useA;
        bool useGradient;
        QString *templateName = 0;
        pfa->readParsTPL(pars, useA, useGradient, &templateName, new QString(*(sst->applicationDirPath) + "/data/templates/" + QString::number(i+1) +".tpl"), this->dst, true);
        if (this->sst->prgset->calcTemplates)
            datacal->calcFitCurve(dst->fitTemplate[i], pars, data->at(0).x(), data->at(data->size() - 1).x(), 1, dst->prgpar->currentWavelength->value, useA, dst->prgpar->par_d, useGradient, dst->prgpar->useCorrAutoColl, dst->prgpar->par_r, dst->prgpar->par_delta, dst->prgpar->par_l, dst->prgpar->useQ, dst->prgpar->par_b);
        else{
            double wavelengthTPL;
            double sampleThicknessTPL;
            if ( pfa->readXYTPL(i, wavelengthTPL, sampleThicknessTPL, new QString(*(sst->applicationDirPath) + "/data/templates/" + QString::number(i+1) +".tpl"), dst) ){
                if ( (wavelengthTPL != dst->prgpar->currentWavelength->value) || ( useA && (sampleThicknessTPL != dst->prgpar->par_d) ) )
                     warning->append(QString::number(i+1) + ", ");
                for (int j = 0; j < dst->fitTemplate[i]->size(); ++j)
                    if (dst->fitTemplate[i]->at(j).x() < data->at(0).x() || dst->fitTemplate[i]->at(j).x() > data->at(data->size() - 1).x()){
                        dst->fitTemplate[i]->remove(j);
                        --j;
                    }
            }
            else
                datacal->calcFitCurve(dst->fitTemplate[i], pars, data->at(0).x(), data->at(data->size() - 1).x(), 1, dst->prgpar->currentWavelength->value, useA, dst->prgpar->par_d, useGradient, dst->prgpar->useCorrAutoColl, dst->prgpar->par_r, dst->prgpar->par_delta, dst->prgpar->par_l, dst->prgpar->useQ, dst->prgpar->par_b);
        }
        curveTemplate[i]->setData(new QwtPointSeriesData(*(dst->fitTemplate[i])));
        plotTemplate[i] = new QwtPlot(this);
        plotTemplate[i]->setFrameShape(QFrame::Box);
        plotTemplate[i]->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        plotTemplate[i]->enableAxis(QwtPlot::yLeft, false);
        plotTemplate[i]->enableAxis(QwtPlot::xBottom, false);
        curveTemplate[i]->attach(plotTemplate[i]);
        plotTemplate[i]->replot();
        labelTemplate[i] = new QLabel(QString::number(i), this, 0);
        labelTemplate[i]->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        labelTemplate[i]->setAlignment(Qt::AlignCenter);
        labelTemplate[i]->setText(QString::number(i+1) + " - " + *templateName);
        delete templateName;
        delete pars;
    }

    plotTemplate[0]->setGeometry(10,40,200,200);
    labelTemplate[0]->setGeometry(10,250,200,20);
    plotTemplate[1]->setGeometry(220,40,200,200);
    labelTemplate[1]->setGeometry(220,250,200,20);
    plotTemplate[2]->setGeometry(430,40,200,200);
    labelTemplate[2]->setGeometry(430,250,200,20);
    plotTemplate[3]->setGeometry(10,290,200,200);
    labelTemplate[3]->setGeometry(10,500,200,20);
    plotTemplate[4]->setGeometry(430,290,200,200);
    labelTemplate[4]->setGeometry(430,500,200,20);
    plotTemplate[5]->setGeometry(10,540,200,200);
    labelTemplate[5]->setGeometry(10,750,200,20);
    plotTemplate[6]->setGeometry(220,540,200,200);
    labelTemplate[6]->setGeometry(220,750,200,20);
    plotTemplate[7]->setGeometry(430,540,200,200);
    labelTemplate[7]->setGeometry(430,750,200,20);

    if (dst->chosenTemplate > 0){
        ui->comboBox_chosenTemplate->setCurrentIndex(dst->chosenTemplate - 1);
        QFont tempFont = labelTemplate[dst->chosenTemplate - 1]->font();
        tempFont.setBold(true);
        labelTemplate[dst->chosenTemplate - 1]->setFont(tempFont);
    }

    if (warning->isEmpty())
        delete warning;
    else{
        warning->chop(2); // remove last ", "
        warning->insert(0, tr("Achtung: Die fuer die Berechnung der X-Y-Daten\nder Vorlagen Nr. "));
        warning->append(tr("\nverwendeten Messparameter stimmen nicht mit den derzeit\nverwendeten Messparametern ueberein. Es wird empfohlen,\ndie Vorlagen mit den derzeit verwendeten Messparametern\nzu berechnen. Soll dies jetzt erfolgen?"));

        QMessageBox *msg = new QMessageBox(this);
        msg->setIcon(QMessageBox::Warning);
        msg->setText(*warning);
        msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
        msg->setDefaultButton(QMessageBox::No);
        int ret = msg->exec();
        switch (ret) {
           case QMessageBox::Yes:
               ui->button_calculateTemplates->click();
               break;
           case QMessageBox::No:
               break;
           default:
               break;
        }
        delete msg;
    }

}

Dialog_chooseTemplate::~Dialog_chooseTemplate(){
    delete ui; //generated by Qt

    curveData->detach();
    delete curveData;
    delete sizeData;
    delete penData;
    delete brushData;
    delete colorData;


    for (int i = 0; i < dst->number_of_templates; ++i){
        dst->fitTemplate[i]->clear();
        curveTemplate[i]->detach();
        delete curveTemplate[i];
        delete plotTemplate[i];
        delete labelTemplate[i];
    }
    delete penTemplate;
    delete colorTemplate;
}


void Dialog_chooseTemplate::keyPressEvent(QKeyEvent *event){
    event->accept();
}

void Dialog_chooseTemplate::closeEvent(QCloseEvent *event){
    if (canClose)
        event->accept();
    else
        event->ignore();
}

void Dialog_chooseTemplate::on_button_chooseTemplate_clicked(){
    dst->chosenTemplate = ui->comboBox_chosenTemplate->currentIndex() + 1;
    if (dst->chosenTemplate > 0){
        canClose = true;
        this->close();
    }
}

void Dialog_chooseTemplate::on_button_chooseNoTemplate_clicked(){
    dst->chosenTemplate = 0;
    canClose = true;
    this->close();
}

void Dialog_chooseTemplate::on_button_calculateTemplates_clicked(){
    ui->button_calculateTemplates->hide();
    for (int i = 0; i < dst->number_of_templates; ++i){
        ParameterSet *pars = new ParameterSet(this);
        bool useA;
        bool useGradient;
        QString *templateName = 0;
        pfa->readParsTPL(pars, useA, useGradient, &templateName, new QString(*(sst->applicationDirPath) + "/data/templates/" + QString::number(i+1) +".tpl"), dst, true);
        dst->fitTemplate[i]->clear();
        datacal->calcFitCurve(dst->fitTemplate[i], pars, data->at(0).x(), data->at(data->size() - 1).x(), 1, dst->prgpar->currentWavelength->value, useA, dst->prgpar->par_d, useGradient, dst->prgpar->useCorrAutoColl, dst->prgpar->par_r, dst->prgpar->par_delta, dst->prgpar->par_l, dst->prgpar->useQ, dst->prgpar->par_b);
        curveTemplate[i]->setData (new QwtPointSeriesData(*(dst->fitTemplate[i])));
        plotTemplate[i]->replot();
        delete templateName;
        delete pars;
    }
}
