/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef DIALOG_PREFERENCES_H
#define DIALOG_PREFERENCES_H

#include <QBrush>
#include <QCloseEvent>
#include <QColor>
#include <QDir>
#include <QFileDialog>
#include <QKeyEvent>
#include <QPen>
#include <QPointF>
#include <QSize>
#include <QString>
#include <QVector>

#include <qwt_curve_fitter.h>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_grid.h>
#include <qwt_series_data.h>
#include <qwt_symbol.h>

#include "enumerations.h"
#include "programsettingsset.h"

#include <QDialog>

namespace Ui {
    class Dialog_Preferences;
}

class Dialog_Preferences : public QDialog{

    Q_OBJECT

public:
    explicit Dialog_Preferences(ProgramSettingsSet *prgset = 0, QWidget *parent = 0);
    ~Dialog_Preferences();

private:
    Ui::Dialog_Preferences *ui;

    bool canClose; //close allowed? (close only by click on one of the buttons)

    ProgramSettingsSet *prgset;

    QVector<QPointF> *dots;
    QVector<QPointF> *line;

    QColor *colorDots;
    QBrush *brushDots;
    QPen *penDots;
    QSize *sizeDots;
    QwtSymbol *symbolDots;
    QwtPlotCurve *curveDots;

    QColor *colorLine;
    QPen *penLine;
    QwtPlotCurve *curveLine;

    QwtPlotGrid *grid;

    void swapDotsLine(Enumerations::drawingOrder order);

    void setWidgetValues();

    //Events
    void keyPressEvent(QKeyEvent *event);
    void closeEvent(QCloseEvent *event);


private slots:

    void on_edit_defaultDir_editingFinished();
    void on_button_choseDefaultDir_clicked();

    void on_spinBox_thicknessOfCurveDots_valueChanged(int arg1);
    void on_spinBox_thicknessOfCurveLines_valueChanged(int arg1);
    void on_button_swapColors_clicked();
    void on_comboBox_elementInFront_currentIndexChanged(int index);

    void on_edit_kernelPath_editingFinished();
    void on_button_choseKernelPath_clicked();

    void on_button_OK_clicked();
    void on_button_Cancel_clicked();
    void on_button_defaultValues_clicked();


};

#endif // DIALOG_PREFERENCES_H
