/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "inifileaccess.h"

INIFileAccess::INIFileAccess(QWidget *parentWidget, QObject *parent) : QObject(parent){
    this->parent = parentWidget;
}

INIFileAccess::~INIFileAccess(){
}

bool INIFileAccess::readProgramSettings(QString *applicationDirPath, ProgramSettingsSet *target){
    bool error = false; //if any parameter cannot be read

    QVariant tempVariant; //for temporary data

    QSettings *settings = new QSettings(*applicationDirPath + "/settings_einstellungen/settings_einstellungen_CarbX.ini", QSettings::IniFormat);

    settings->sync();

    //BEGIN DO NOT TRANSLATE

    settings->beginGroup("program-settings");

    tempVariant = settings->value("simpleMode");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        target->simpleMode = tempVariant.toBool();
    else
        error = true;

    tempVariant = settings->value("autoUpdateFit");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        target->autoUpdateFit = tempVariant.toBool();
    else
        error = true;

    tempVariant = settings->value("defaultDir");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::String)){
        delete target->defaultDir;
        target->defaultDir = new QString(tempVariant.toString());
    }
    else
        error = true;

    tempVariant = settings->value("displayFullPath");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        target->displayFullPath = tempVariant.toBool();
    else
        error = true;

    tempVariant = settings->value("adjustAxisIntervals");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        target->adjustAxisIntervals = tempVariant.toBool();
    else
        error = true;

    tempVariant = settings->value("thicknessOfCurveDots");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Int))
        target->thicknessOfCurveDots = tempVariant.toInt();
    else
        error = true;

    tempVariant = settings->value("thicknessOfCurveLines");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Int))
        target->thicknessOfCurveLines = tempVariant.toInt();
    else
        error = true;

    tempVariant = settings->value("elementInFront");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Int))
        target->elementInFront = Enumerations::drawingOrder(tempVariant.toInt());
    else
        error = true;

    tempVariant = settings->value("enableChooseTemplate");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        target->enableChooseTemplate = tempVariant.toBool();
    else
        error = true;

    tempVariant = settings->value("calcTemplates");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Bool))
        target->calcTemplates = tempVariant.toBool();
    else
        error = true;

    tempVariant = settings->value("kernelPath");
    if (tempVariant.isValid() && tempVariant.canConvert(QVariant::String)){
        delete target->kernelPath;
        target->kernelPath = new QString(tempVariant.toString());
    }
    else
        error = true;

    settings->endGroup();

    //END DO NOT TRANSLATE

    if ( settings->status() != QSettings::NoError)
        error = true;

    delete settings;
    return !error;
}

bool INIFileAccess::writeProgramSettings(QString *applicationDirPath, ProgramSettingsSet *target){
    bool error = false; //if any error occurs

    QSettings *settings = new QSettings(*applicationDirPath + "/settings_einstellungen/settings_einstellungen_CarbX.ini", QSettings::IniFormat);

    settings->clear();
    settings->sync();

    //BEGIN DO NOT TRANSLATE

    settings->beginGroup("WARNING");
    settings->setValue("WARNING1", "---This file must not be changed manually!---");
    settings->setValue("WARNING2", "---Manual changes can cause program crashes and data loss when loading the file!---");
    settings->endGroup();

    settings->beginGroup("WARNUNG");
    settings->setValue("WARNUNG1", "---Diese Datei darf nicht manuell veraendert werden!---");
    settings->setValue("WARNUNG2", "---Manuelle Aenderungen koennen beim Laden der Datei Abstuerze und Datenverlust verursachen!---");
    settings->endGroup();

    settings->beginGroup("program-settings");
    settings->setValue("simpleMode",target->simpleMode);
    settings->setValue("autoUpdateFit", target->autoUpdateFit);
    settings->setValue("defaultDir", *(target->defaultDir));
    settings->setValue("displayFullPath", target->displayFullPath);
    settings->setValue("adjustAxisIntervals", target->adjustAxisIntervals);
    settings->setValue("thicknessOfCurveDots", target->thicknessOfCurveDots);
    settings->setValue("thicknessOfCurveLines", target->thicknessOfCurveLines);
    settings->setValue("elementInFront", target->elementInFront);
    settings->setValue("enableChooseTemplate", target->enableChooseTemplate);
    settings->setValue("calcTemplates", target->calcTemplates);
    settings->setValue("kernelPath", *(target->kernelPath));
    settings->endGroup();

    //END DO NOT TRANSLATE

    if (settings->isWritable())
        settings->sync();
    else
        error = true;

    if ( settings->status() != QSettings::NoError)
        error = true;

    delete settings;
    return !error;
}

bool INIFileAccess::readParsDefaultValues(ParameterSet *target, QString *applicationDirPath, QVector<parsMean> *parsMeaning, QVector<QString> *parsType){
    bool error = false; //if any parameter cannot be read

    QVariant tempVariant; //for temporary data

    QSettings *settings = new QSettings( *applicationDirPath + tr("/data/pars-defaultValues_de.txt"), QSettings::IniFormat);

    settings->sync();

    for (int i = 0; i < Enumerations::countOf_parName; ++i){
        settings->beginGroup(parsMeaning->at(i).symbol);
        for (int j = 0; j < Enumerations::countOf_parType; ++j){
            tempVariant = settings->value(parsType->at(j));
            if (tempVariant.isValid() && tempVariant.canConvert(QVariant::Double))
                target->setParameterValue(Enumerations::parName(i), Enumerations::parType(j), tempVariant.toDouble(), false);
            else
                error = true;
        }
        settings->endGroup();
    }

    if ( ! ( (settings->status() == QSettings::NoError) && (QFile::exists(*applicationDirPath + tr("/data/pars-defaultValues_de.txt")) ) ) )
        error = true;
    delete settings;
    return !error;
}
