/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef PARAMETERFILEACCESS_H
#define PARAMETERFILEACCESS_H

#include <QFile>
#include <QFileDialog>
#include <QPointF>
#include <QSettings>
#include <QString>
#include <QStringList>
#include <QVariant>
#include <QVector>

#include "autofitsequence.h"
#include "autofitstep.h"
#include "auxiliarymethods.h"
#include "datastorage.h"
#include "datacalculations.h"
#include "dialog_name.h"
#include "enumerations.h"
#include "settingsstorage.h"
#include "structs.h"

#include <QObject>

class ParameterFileAccess : public QObject{

    Q_OBJECT

public:
    explicit ParameterFileAccess(DataCalculations *datacal, QWidget *parentWidget = 0, QObject *parent = 0);
    ~ParameterFileAccess();

    Enumerations::errorType loadPars       (DataStorage *dst, SettingsStorage *sst);
    bool                    readPars       (QString *readPath, DataStorage *dst, SettingsStorage *sst);
    bool                    readSliderPars (QString *readPath, DataStorage *dst); //only reads pars for sliders

    bool savePars  (DataStorage *dst, SettingsStorage *sst, int scrollBarControls_value);
    bool writePars (QString *writePath, DataStorage *dst, SettingsStorage *sst, int scrollBarControls_value);

    bool readParsTPL (ParameterSet *target, bool &useA, bool &useGradient, QString **templateName, QString *readPath, DataStorage *dst, bool setDefaultValues); //reads template file
    bool readXYTPL (int index, double &wavelength_value, double &sample_thickness, QString *readPath, DataStorage *dst); //reads X-Y data of template file

    bool saveParsTPL  (DataStorage *dst, QString *defaultDir);
    bool writeParsTPL (QString *writePath, QString *templateName, DataStorage *dst);

    bool readAFS         (AutoFitSequence *target, QString *readPath, DataStorage *dst);
    QString* readAFSName (QString *readPath); //reads only template Name
    bool writeAFS(AutoFitSequence *toWrite, QString *writePath, DataStorage *dst);

private:
    QWidget *parent;
    DataCalculations *datacal;


};

#endif // PARAMETERFILEACCESS_H
