/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#include "programparameterset.h"

ProgramParameterSet::ProgramParameterSet(QWidget *parentWidget, QObject *parent) : QObject(parent){
    this->parent = parentWidget;
    currentWavelength = 0;
}

ProgramParameterSet::~ProgramParameterSet(){
    delete currentWavelength;
}

void ProgramParameterSet::setDefaultValues(bool adjustAxisIntervals){
    if (adjustAxisIntervals){
        plotMinX = 0.01;
        plotMaxX = (floor(( (2 * sin(70 * M_PI / 180)) / (currentWavelength->value) ) * 10.) / 10.);
        plotMinY = 3;
        plotMaxY = 5;
        autoScaleYAxis = true;
    }

    par_r = 24.;
    par_d = 0.1;
    par_b = 0.002;
    par_delta = 0.5;
    par_l = 0.7;

    useQ = false;
    useA = false;
    useGradient = false;
    useCorrAutoColl = false;

    corrPosition = false;
    corrPositionMethod = Enumerations::Dz;

    corrIntensity = false;
    corrIntensityMethod = Enumerations::LambertBeer;

    XFormat = Enumerations::twotheta_degree;
    YFormat = Enumerations::linear;

    thinningFactor = 10;

    //deviation window settings
    showInterpolation = false;
    followChanges = false;
}

