/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef PROGRAMPARAMETERSET_H
#define PROGRAMPARAMETERSET_H

#include <cmath>

#include "enumerations.h"
#include "structs.h"

#include <QObject>

class ProgramParameterSet : public QObject{

    Q_OBJECT

public:
    explicit ProgramParameterSet(QWidget *parentWidget, QObject *parent = 0);
    ~ProgramParameterSet();

    //most variables public for easier access

    wavelength *currentWavelength;

    double plotMinX;
    double plotMaxX;
    double plotMinY;
    double plotMaxY;
    bool autoScaleYAxis;

    double par_r; //radius of diffractometer
    double par_d; //thickness of sample
    double par_b; //pass-band b
    double par_delta; //divergence angle
    double par_l; //irradiated length

    bool useQ; //sec. monochromation
    bool useA; //factor for absorption
    bool useGradient; //tuning of gradient/slope
    bool useCorrAutoColl; //correct automatic collimator

    //data correction - position
    bool corrPosition;
    Enumerations::corrPositionMethod corrPositionMethod;

    //data correction - intensity
    bool corrIntensity;
    Enumerations::corrIntensityMethod corrIntensityMethod;

    //format of raw data
    Enumerations::XFormat XFormat;
    Enumerations::YFormat YFormat;

    //thinning factor
    int thinningFactor; //also used to restore thinning factor when leaving simple mode

    //deviation window settings
    bool showInterpolation;
    bool followChanges;


    //public methods
    void setDefaultValues(bool adjustAxisIntervals); //must be called after construction


private:
    QWidget *parent;

};

#endif // PROGRAMPARAMETERSET_H
