/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef PROGRAMSETTINGSSET_H
#define PROGRAMSETTINGSSET_H

#include <QString>

#include "enumerations.h"

#include <QObject>

class ProgramSettingsSet : public QObject{

    Q_OBJECT

public:

    //is initialised with default values for public and private variables
    explicit ProgramSettingsSet(QWidget *parentWidget = 0, QObject *parent = 0);

    ~ProgramSettingsSet();

    //get methods for private stored default program settings
    bool get_simpleMode_default();
    bool get_autoUpdateFit_default();
    QString get_defaultDir_default() const;
    bool get_displayFullPath_default();
    bool get_adjustAxisIntervals_default();
    short get_thicknessOfCurveDots_default();
    short get_thicknessOfCurveLines_default();
    Enumerations::drawingOrder get_elementInFront_default();
    bool get_enableChooseTemplate_default();
    bool get_calcTemplates_default();
    QString get_kernelPath_default() const;


    //global program settings - variables are public for easier access
    bool simpleMode;
    bool autoUpdateFit;
    QString *defaultDir;
    bool displayFullPath;
    bool adjustAxisIntervals;
    short thicknessOfCurveDots;
    short thicknessOfCurveLines;
    Enumerations::drawingOrder elementInFront;
    bool enableChooseTemplate;
    bool calcTemplates;
    QString *kernelPath;

private:
    QWidget *parent;

    /* default global program settings
     * used for initialisation
     * needed to restore settings on "Cancel" after "Default Values" in Dialog_Preferences
     */
    bool simpleMode_default;
    bool autoUpdateFit_default;
    QString *defaultDir_default;
    bool displayFullPath_default;
    bool adjustAxisIntervals_default;
    short thicknessOfCurveDots_default;
    short thicknessOfCurveLines_default;
    Enumerations::drawingOrder elementInFront_default;
    bool enableChooseTemplate_default;
    bool calcTemplates_default;
    QString *kernelPath_default;

};


#endif // PROGRAMSETTINGSSET_H
