/*
 * CarbX, except for the proprietary licensed third party source code in human
 * and/or machine-readable form, is free software. The third-party source code is
 * not shipped in human and/or machine-readable form along with these source
 * files. If you build CarbX from the provided CarbX source code files while
 * obtaining the required libraries, files and third party source code as well as
 * the required licences on your own, you can redistribute and/or modify the
 * remaining parts of CarbX (i. e. the parts developed by Torben Pfaff) under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version. This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see http://www.gnu.org/licenses/.
 */

#ifndef SETTINGSSTORAGE_H
#define SETTINGSSTORAGE_H

#include <QPoint>

#include "inifileaccess.h"
#include "programsettingsset.h"

#include <QObject>

class SettingsStorage : public QObject{

    Q_OBJECT

public:
    explicit SettingsStorage(QString *applicationDirPath = 0, QWidget *parentWidget = 0, QObject *parent = 0);
    ~SettingsStorage();

    //most variables are public for easier access

    QString *applicationDirPath; //path to CarbX.exe

    //global program settings
    ProgramSettingsSet *prgset;

    //program states
    bool drawData;
    bool drawFit;
    bool drawDev; //deviation

    //auxialiary variables for program functions
    bool mustSave;

    //variables used for temporary storage only
    int scrollBarControls_Value_temp;

    //variables for specific widget are to be initialised/deleted in its constructor/destructor
    QPoint *minSize_MainWindow; //minimal size for MainWindow
    int visibleSlider_MainWindow;
    bool checkParameters_MainWindow; //should parameters set via slots of GUI elements be checked, default true but no need to do so if parameter files are read
    bool reCheck_buttonShowDeviation_MainWindow; //auxiliary variable for reshow deviation when it was closed due to hiding measurement curve / fit curve
    bool reCheck_optA_MainWindow; //auxiliary variable for alternating enabling of A/corr. of intensity
    bool reCheck_optIntensity_MainWindow; //auxiliary variable for alternating enabling of A/corr. of intensity

    //variable for specific widget are to be initialised/deleted in its constructor/destructor
    QPoint *minSize_Deviation;


    //public methods
    readProgramSettings();
    writeProgramSettings();

private:
    QWidget *parent;
    INIFileAccess *ifa;
};

#endif // SETTINGSSTORAGE_H
